package com.biganto.visual.roompark.data.data_provider

import android.app.Application
import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.file.FileModule
import com.biganto.visual.roompark.domain.contract.FilesContract
import com.biganto.visual.roompark.util.extensions.folderSize
import timber.log.Timber
import java.io.File
import javax.inject.Inject
import kotlin.reflect.full.primaryConstructor

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */

//@Singleton
class FilesContractModule @Inject constructor(
    private val files: FileModule,
    private val api: IRoomParkApi,
    private val db: IDb,
    private val context: Application
): FilesContract {


    private inline fun <reified T:FileModule.FileDirectory> getDirectory(): T? {
        val type = T::class
        Timber.d("Type is :$type")
        val inst = type.primaryConstructor?.call(null,null)
        Timber.d("inst is :$inst")
        //val inst2 = type.constructors.first().call()
        //Timber.d("inst2 is :$inst2")
        return inst
    }


    private inline fun <reified T:FileModule.FileDirectory> getFile(): File {
        Timber.d("type: ${T::class}")
        val d = getDirectory<T>()
        Timber.d("d: ${d}")
        if (d==null) error("Unresolved class type")
        return FileModule.getDirectory(context,d)
    }

    override fun getPlansSize() = getFile<FileModule.FileDirectory.PlanTypeDir>().folderSize

    override fun getToursSize() = getFile<FileModule.FileDirectory.ToursDir>().folderSize

    override fun getFeedSize() = getFile<FileModule.FileDirectory.FeedsDir>().folderSize

    override fun getAlbumSize() = getFile<FileModule.FileDirectory.Albums>().folderSize

    override fun allCacheSize() = FileModule.assetsFile(context).folderSize

}

